@@ -6,6 +6,7 @@ import asyncio
 import itertools
 import logging
 from collections.abc import Callable, Iterable
+from dataclasses import asdict
 from functools import partial
 from typing import TYPE_CHECKING, Any, Final
 
@@ -21,6 +22,7 @@ from bluetooth_adapters import (
     ADAPTER_PASSIVE_SCAN,
     AdapterDetails,
     BluetoothAdapters,
+    get_adapters,
 )
 from bluetooth_data_tools import monotonic_time_coarse
 
@@ -35,7 +37,12 @@ from .const import (
     FALLBACK_MAXIMUM_STALE_ADVERTISEMENT_SECONDS,
     UNAVAILABLE_TRACK_SECONDS,
 )
-from .models import BluetoothServiceInfoBleak, HaBluetoothSlotAllocations
+from .models import (
+    BluetoothServiceInfoBleak,
+    HaBluetoothSlotAllocations,
+    HaScannerRegistration,
+    HaScannerRegistrationEvent,
+)
 from .scanner_device import BluetoothScannerDevice
 from .usage import install_multiple_bleak_catcher, uninstall_multiple_bleak_catcher
 from .util import async_reset_adapter
@@ -54,11 +61,13 @@ APPLE_IBEACON_START_BYTE: Final = 0x02  # iBeacon (tilt_ble)
 APPLE_HOMEKIT_START_BYTE: Final = 0x06  # homekit_controller
 APPLE_DEVICE_ID_START_BYTE: Final = 0x10  # bluetooth_le_tracker
 APPLE_HOMEKIT_NOTIFY_START_BYTE: Final = 0x11  # homekit_controller
+APPLE_FINDMY_START_BYTE: Final = 0x12  # FindMy network advertisements
 APPLE_START_BYTES_WANTED: Final = {
     APPLE_IBEACON_START_BYTE,
     APPLE_HOMEKIT_START_BYTE,
     APPLE_HOMEKIT_NOTIFY_START_BYTE,
     APPLE_DEVICE_ID_START_BYTE,
+    APPLE_FINDMY_START_BYTE,
 }
 
 _str = str
@@ -121,6 +130,7 @@ class BluetoothManager:
         "_loop",
         "_non_connectable_scanners",
         "_recovery_lock",
+        "_scanner_registration_callbacks",
         "_sources",
         "_unavailable_callbacks",
         "shutdown",
@@ -129,8 +139,8 @@ class BluetoothManager:
 
     def __init__(
         self,
-        bluetooth_adapters: BluetoothAdapters,
-        slot_manager: BleakSlotManager,
+        bluetooth_adapters: BluetoothAdapters | None = None,
+        slot_manager: BleakSlotManager | None = None,
     ) -> None:
         """Init bluetooth manager."""
         self._cancel_unavailable_tracking: asyncio.TimerHandle | None = None
@@ -155,8 +165,8 @@ class BluetoothManager:
         self._adapter_sources: dict[str, str] = {}
         self._allocations: dict[str, HaBluetoothSlotAllocations] = {}
         self._sources: dict[str, BaseHaScanner] = {}
-        self._bluetooth_adapters = bluetooth_adapters
-        self.slot_manager = slot_manager
+        self._bluetooth_adapters = bluetooth_adapters or get_adapters()
+        self.slot_manager = slot_manager or BleakSlotManager()
         self._cancel_allocation_callbacks = (
             self.slot_manager.register_allocation_callback(
                 self._async_slot_manager_changed
@@ -171,6 +181,9 @@ class BluetoothManager:
         self._allocations_callbacks: dict[
             str | None, set[Callable[[HaBluetoothSlotAllocations], None]]
         ] = {}
+        self._scanner_registration_callbacks: dict[
+            str | None, set[Callable[[HaScannerRegistration], None]]
+        ] = {}
 
     @property
     def supports_passive_scan(self) -> bool:
@@ -196,6 +209,10 @@ class BluetoothManager:
         return {
             "adapters": self._adapters,
             "slot_manager": self.slot_manager.diagnostics(),
+            "allocations": {
+                source: asdict(allocations)
+                for source, allocations in self._allocations.items()
+            },
             "scanners": scanner_diagnostics,
             "connectable_history": [
                 service_info.as_dict()
@@ -283,6 +300,10 @@ class BluetoothManager:
 
     async def async_setup(self) -> None:
         """Set up the bluetooth manager."""
+        from .central_manager import CentralBluetoothManager
+
+        if CentralBluetoothManager.manager is None:
+            CentralBluetoothManager.manager = self
         self._loop = asyncio.get_running_loop()
         await self._async_refresh_adapters()
         install_multiple_bleak_catcher()
@@ -489,7 +510,6 @@ class BluetoothManager:
             old_connectable_service_info = self._connectable_history.get(address)
         else:
             old_connectable_service_info = None
-        source = service_info.source
         # This logic is complex due to the many combinations of scanners
         # that are supported.
         #
@@ -508,8 +528,8 @@ class BluetoothManager:
         #
         if (
             (old_service_info := self._all_history.get(address)) is not None
-            and source != (old_source := old_service_info.source)
-            and (scanner := self._sources.get(old_source)) is not None
+            and service_info.source != old_service_info.source
+            and (scanner := self._sources.get(old_service_info.source)) is not None
             and scanner.scanning
             and self._prefer_previous_adv_from_different_source(
                 address, old_service_info, service_info
@@ -528,11 +548,10 @@ class BluetoothManager:
                     # source, we need to check it as well to see if we prefer
                     # the old connectable advertisement
                     or (
-                        (old_connectable_source := old_connectable_service_info.source)
-                        != source
+                        old_connectable_service_info.source != service_info.source
                         and (
                             connectable_scanner := self._sources.get(
-                                old_connectable_source
+                                old_connectable_service_info.source
                             )
                         )
                         is not None
@@ -555,14 +574,13 @@ class BluetoothManager:
 
         # Track advertisement intervals to determine when we need to
         # switch adapters or mark a device as unavailable
-        tracker = self._advertisement_tracker
         if (
-            last_source := tracker.sources.get(address)
-        ) is not None and last_source != source:
+            last_source := self._advertisement_tracker.sources.get(address)
+        ) is not None and last_source != service_info.source:
             # Source changed, remove the old address from the tracker
-            tracker.async_remove_address(address)
-        if address not in tracker.intervals:
-            tracker.async_collect(service_info)
+            self._advertisement_tracker.async_remove_address(address)
+        if address not in self._advertisement_tracker.intervals:
+            self._advertisement_tracker.async_collect(service_info)
 
         # If the advertisement data is the same as the last time we saw it, we
         # don't need to do anything else unless its connectable and we are missing
@@ -587,20 +605,7 @@ class BluetoothManager:
             # route any connection attempts to the connectable path, we
             # mark the service_info as connectable so that the callbacks
             # will be called and the device can be discovered.
-            service_info = BluetoothServiceInfoBleak(
-                service_info.name,
-                address,
-                service_info.rssi,
-                service_info.manufacturer_data,
-                service_info.service_data,
-                service_info.service_uuids,
-                source,
-                service_info.device,
-                service_info._advertisement,
-                True,
-                service_info.time,
-                service_info.tx_power,
-            )
+            service_info = service_info._as_connectable()
 
         if (connectable or old_connectable_service_info is not None) and (
             bleak_callbacks := self._bleak_callbacks
@@ -708,6 +713,7 @@ class BluetoothManager:
         self._allocations.pop(scanner.source, None)
         if connection_slots:
             self.slot_manager.remove_adapter(scanner.adapter)
+        self._async_on_scanner_registration(scanner, HaScannerRegistrationEvent.REMOVED)
 
     def async_register_scanner(
         self,
@@ -728,6 +734,10 @@ class BluetoothManager:
         self._adapter_sources[scanner.adapter] = scanner.source
         if connection_slots:
             self.slot_manager.register_adapter(scanner.adapter, connection_slots)
+            self.async_on_allocation_changed(
+                self.slot_manager.get_allocations(scanner.adapter)
+            )
+        self._async_on_scanner_registration(scanner, HaScannerRegistrationEvent.ADDED)
         return partial(
             self._async_unregister_scanner_internal, scanners, scanner, connection_slots
         )
@@ -797,6 +807,23 @@ class BluetoothManager:
                 except Exception:
                     _LOGGER.exception("Error in allocation callback")
 
+    def _async_on_scanner_registration(
+        self, scanner: BaseHaScanner, event: HaScannerRegistrationEvent
+    ) -> None:
+        """Call scanner callbacks."""
+        for source_key in (scanner.source, None):
+            if not (
+                scanner_callbacks := self._scanner_registration_callbacks.get(
+                    source_key
+                )
+            ):
+                continue
+            for callback_ in scanner_callbacks:
+                try:
+                    callback_(HaScannerRegistration(event, scanner))
+                except Exception:
+                    _LOGGER.exception("Error in scanner callback")
+
     def async_current_allocations(
         self, source: str | None = None
     ) -> list[HaBluetoothSlotAllocations] | None:
@@ -823,3 +850,24 @@ class BluetoothManager:
             callbacks.discard(callback)
             if not callbacks:
                 del self._allocations_callbacks[source]
+
+    def async_register_scanner_registration_callback(
+        self, callback: Callable[[HaScannerRegistration], None], source: str | None
+    ) -> CALLBACK_TYPE:
+        """Register a callback to be called when a scanner is added or removed."""
+        self._scanner_registration_callbacks.setdefault(source, set()).add(callback)
+        return partial(
+            self._async_unregister_scanner_registration_callback, callback, source
+        )
+
+    def _async_unregister_scanner_registration_callback(
+        self, callback: Callable[[HaScannerRegistration], None], source: str | None
+    ) -> None:
+        if (callbacks := self._scanner_registration_callbacks.get(source)) is not None:
+            callbacks.discard(callback)
+            if not callbacks:
+                del self._scanner_registration_callbacks[source]
+
+    def async_current_scanners(self) -> list[BaseHaScanner]:
+        """Return the current scanners."""
+        return list(self._sources.values())
