@@ -13,7 +13,7 @@ from bleak.backends.scanner import AdvertisementData
 from bleak_retry_connector import NO_RSSI_VALUE
 
 if TYPE_CHECKING:
-    from .manager import BluetoothManager
+    from .base_scanner import BaseHaScanner
 
 _BluetoothServiceInfoSelfT = TypeVar(
     "_BluetoothServiceInfoSelfT", bound="BluetoothServiceInfo"
@@ -29,24 +29,6 @@ _str = str  # avoid cython conversion since we always want a pystr
 _int = int  # avoid cython conversion since we always want a pyint
 
 
-class CentralBluetoothManager:
-    """Central Bluetooth Manager."""
-
-    manager: BluetoothManager | None = None
-
-
-def get_manager() -> BluetoothManager:
-    """Get the BluetoothManager."""
-    if TYPE_CHECKING:
-        assert CentralBluetoothManager.manager is not None
-    return CentralBluetoothManager.manager
-
-
-def set_manager(manager: BluetoothManager) -> None:
-    """Set the BluetoothManager."""
-    CentralBluetoothManager.manager = manager
-
-
 @dataclass(slots=True, frozen=True)
 class HaBluetoothSlotAllocations:
     """Data for how to allocate slots for BLEDevice connections."""
@@ -57,6 +39,22 @@ class HaBluetoothSlotAllocations:
     allocated: list[str]  # Addresses of connected devices
 
 
+class HaScannerRegistrationEvent(Enum):
+    """Events for scanner registration."""
+
+    ADDED = "added"
+    REMOVED = "removed"
+    UPDATED = "updated"
+
+
+@dataclass(slots=True, frozen=True)
+class HaScannerRegistration:
+    """Data for a scanner event."""
+
+    event: HaScannerRegistrationEvent
+    scanner: BaseHaScanner
+
+
 @dataclass(slots=True)
 class HaBluetoothConnector:
     """Data for how to connect a BLEDevice from a given scanner."""
@@ -66,6 +64,16 @@ class HaBluetoothConnector:
     can_connect: Callable[[], bool]
 
 
+@dataclass(slots=True, frozen=True)
+class HaScannerDetails:
+    """Details for a scanner."""
+
+    source: str
+    connectable: bool
+    name: str
+    adapter: str
+
+
 class BluetoothScanningMode(Enum):
     """The mode of scanning for bluetooth devices."""
 
@@ -289,3 +297,20 @@ class BluetoothServiceInfoBleak(BluetoothServiceInfo):
             time,
             advertisement_data.tx_power,
         )
+
+    def _as_connectable(self) -> BluetoothServiceInfoBleak:
+        """Return a connectable version of this object."""
+        new_obj = BluetoothServiceInfoBleak.__new__(BluetoothServiceInfoBleak)
+        new_obj.name = self.name
+        new_obj.address = self.address
+        new_obj.rssi = self.rssi
+        new_obj.manufacturer_data = self.manufacturer_data
+        new_obj.service_data = self.service_data
+        new_obj.service_uuids = self.service_uuids
+        new_obj.source = self.source
+        new_obj.device = self.device
+        new_obj._advertisement = self._advertisement
+        new_obj.connectable = True
+        new_obj.time = self.time
+        new_obj.tx_power = self.tx_power
+        return new_obj
